@@ -121,6 +121,8 @@ pub struct ReaderOptions {
 ///
 /// Return inferred schema and number of records used for inference. This function does not change
 /// reader cursor offset.
+///
+/// The inferred schema will always have each field set as nullable.
 pub fn infer_file_schema<R: Read + Seek>(
     reader: R,
     delimiter: u8,
@@ -200,8 +202,6 @@ fn infer_reader_schema_with_csv_options<R: Read>(
     let header_length = headers.len();
     // keep track of inferred field types
     let mut column_types: Vec<HashSet<DataType>> = vec![HashSet::new(); header_length];
-    // keep track of columns with nulls
-    let mut nulls: Vec<bool> = vec![false; header_length];
 
     let mut records_count = 0;
     let mut fields = vec![];
@@ -214,12 +214,12 @@ fn infer_reader_schema_with_csv_options<R: Read>(
         }
         records_count += 1;
 
-        for i in 0..header_length {
+        // Note since we may be looking at a sample of the data, we make the safe assumption that
+        // they could be nullable
+        for (i, column_type) in column_types.iter_mut().enumerate().take(header_length) {
             if let Some(string) = record.get(i) {
-                if string.is_empty() {
-                    nulls[i] = true;
-                } else {
-                    column_types[i]
+                if !string.is_empty() {
+                    column_type
                         .insert(infer_field_schema(string, roptions.datetime_re.clone()));
                 }
             }
@@ -229,7 +229,6 @@ fn infer_reader_schema_with_csv_options<R: Read>(
     // build schema from inference results
     for i in 0..header_length {
         let possibilities = &column_types[i];
-        let has_nulls = nulls[i];
         let field_name = &headers[i];
 
         // determine data type based on possible types
@@ -237,7 +236,7 @@ fn infer_reader_schema_with_csv_options<R: Read>(
         match possibilities.len() {
             1 => {
                 for dtype in possibilities.iter() {
-                    fields.push(Field::new(field_name, dtype.clone(), has_nulls));
+                    fields.push(Field::new(field_name, dtype.clone(), true));
                 }
             }
             2 => {
@@ -245,13 +244,13 @@ fn infer_reader_schema_with_csv_options<R: Read>(
                     && possibilities.contains(&DataType::Float64)
                 {
                     // we have an integer and double, fall down to double
-                    fields.push(Field::new(field_name, DataType::Float64, has_nulls));
+                    fields.push(Field::new(field_name, DataType::Float64, true));
                 } else {
                     // default to Utf8 for conflicting datatypes (e.g bool and int)
-                    fields.push(Field::new(field_name, DataType::Utf8, has_nulls));
+                    fields.push(Field::new(field_name, DataType::Utf8, true));
                 }
             }
-            _ => fields.push(Field::new(field_name, DataType::Utf8, has_nulls)),
+            _ => fields.push(Field::new(field_name, DataType::Utf8, true)),
         }
     }
 
@@ -708,8 +707,7 @@ fn build_decimal_array(
     precision: u8,
     scale: u8,
 ) -> Result<ArrayRef> {
-    let mut decimal_builder =
-        Decimal128Builder::with_capacity(rows.len(), precision, scale);
+    let mut decimal_builder = Decimal128Builder::with_capacity(rows.len());
     for row in rows {
         let col_s = row.get(col_idx);
         match col_s {
@@ -726,7 +724,7 @@ fn build_decimal_array(
                         parse_decimal_with_parameter(s, precision, scale);
                     match decimal_value {
                         Ok(v) => {
-                            decimal_builder.append_value(v)?;
+                            decimal_builder.append_value(v);
                         }
                         Err(e) => {
                             return Err(e);
@@ -736,7 +734,11 @@ fn build_decimal_array(
             }
         }
     }
-    Ok(Arc::new(decimal_builder.finish()))
+    Ok(Arc::new(
+        decimal_builder
+            .finish()
+            .with_precision_and_scale(precision, scale)?,
+    ))
 }
 
 // Parse the string format decimal value to i128 format and checking the precision and scale.
@@ -1136,7 +1138,7 @@ mod tests {
     use crate::array::*;
     use crate::compute::cast;
     use crate::datatypes::Field;
-    use chrono::{prelude::*, LocalResult};
+    use chrono::prelude::*;
 
     #[test]
     fn test_csv() {
@@ -1287,9 +1289,9 @@ mod tests {
 
         let mut csv = builder.build(file).unwrap();
         let expected_schema = Schema::new(vec![
-            Field::new("city", DataType::Utf8, false),
-            Field::new("lat", DataType::Float64, false),
-            Field::new("lng", DataType::Float64, false),
+            Field::new("city", DataType::Utf8, true),
+            Field::new("lat", DataType::Float64, true),
+            Field::new("lng", DataType::Float64, true),
         ]);
         assert_eq!(Arc::new(expected_schema), csv.schema());
         let batch = csv.next().unwrap().unwrap();
@@ -1514,10 +1516,10 @@ mod tests {
             ]
         );
 
-        assert!(!schema.field(0).is_nullable());
+        assert!(schema.field(0).is_nullable());
         assert!(schema.field(1).is_nullable());
         assert!(schema.field(2).is_nullable());
-        assert!(!schema.field(3).is_nullable());
+        assert!(schema.field(3).is_nullable());
         assert!(schema.field(4).is_nullable());
         assert!(schema.field(5).is_nullable());
 
@@ -1696,26 +1698,6 @@ mod tests {
         }
     }
 
-    /// Interprets a naive_datetime (with no explicit timezone offset)
-    /// using the local timezone and returns the timestamp in UTC (0
-    /// offset)
-    fn naive_datetime_to_timestamp(naive_datetime: &NaiveDateTime) -> i64 {
-        // Note: Use chrono APIs that are different than
-        // naive_datetime_to_timestamp to compute the utc offset to
-        // try and double check the logic
-        let utc_offset_secs = match Local.offset_from_local_datetime(naive_datetime) {
-            LocalResult::Single(local_offset) => {
-                local_offset.fix().local_minus_utc() as i64
-            }
-            _ => panic!(
-                "Unexpected failure converting {} to local datetime",
-                naive_datetime
-            ),
-        };
-        let utc_offset_nanos = utc_offset_secs * 1_000_000_000;
-        naive_datetime.timestamp_nanos() - utc_offset_nanos
-    }
-
     #[test]
     fn test_parse_timestamp_microseconds() {
         assert_eq!(
@@ -1728,11 +1710,11 @@ mod tests {
         );
         assert_eq!(
             parse_item::<TimestampMicrosecondType>("2018-11-13T17:11:10").unwrap(),
-            naive_datetime_to_timestamp(&naive_datetime) / 1000
+            naive_datetime.timestamp_nanos() / 1000
         );
         assert_eq!(
             parse_item::<TimestampMicrosecondType>("2018-11-13 17:11:10").unwrap(),
-            naive_datetime_to_timestamp(&naive_datetime) / 1000
+            naive_datetime.timestamp_nanos() / 1000
         );
         let naive_datetime = NaiveDateTime::new(
             NaiveDate::from_ymd(2018, 11, 13),
@@ -1740,7 +1722,7 @@ mod tests {
         );
         assert_eq!(
             parse_item::<TimestampMicrosecondType>("2018-11-13T17:11:10.011").unwrap(),
-            naive_datetime_to_timestamp(&naive_datetime) / 1000
+            naive_datetime.timestamp_nanos() / 1000
         );
         let naive_datetime = NaiveDateTime::new(
             NaiveDate::from_ymd(1900, 2, 28),
@@ -1748,7 +1730,7 @@ mod tests {
         );
         assert_eq!(
             parse_item::<TimestampMicrosecondType>("1900-02-28T12:34:56").unwrap(),
-            naive_datetime_to_timestamp(&naive_datetime) / 1000
+            naive_datetime.timestamp_nanos() / 1000
         );
     }
 
@@ -1764,11 +1746,11 @@ mod tests {
         );
         assert_eq!(
             parse_item::<TimestampNanosecondType>("2018-11-13T17:11:10").unwrap(),
-            naive_datetime_to_timestamp(&naive_datetime)
+            naive_datetime.timestamp_nanos()
         );
         assert_eq!(
             parse_item::<TimestampNanosecondType>("2018-11-13 17:11:10").unwrap(),
-            naive_datetime_to_timestamp(&naive_datetime)
+            naive_datetime.timestamp_nanos()
         );
         let naive_datetime = NaiveDateTime::new(
             NaiveDate::from_ymd(2018, 11, 13),
@@ -1776,7 +1758,7 @@ mod tests {
         );
         assert_eq!(
             parse_item::<TimestampNanosecondType>("2018-11-13T17:11:10.011").unwrap(),
-            naive_datetime_to_timestamp(&naive_datetime)
+            naive_datetime.timestamp_nanos()
         );
         let naive_datetime = NaiveDateTime::new(
             NaiveDate::from_ymd(1900, 2, 28),
@@ -1784,7 +1766,7 @@ mod tests {
         );
         assert_eq!(
             parse_item::<TimestampNanosecondType>("1900-02-28T12:34:56").unwrap(),
-            naive_datetime_to_timestamp(&naive_datetime)
+            naive_datetime.timestamp_nanos()
         );
     }
 
@@ -1818,10 +1800,10 @@ mod tests {
         )?;
 
         assert_eq!(schema.fields().len(), 4);
-        assert!(!schema.field(0).is_nullable());
+        assert!(schema.field(0).is_nullable());
         assert!(schema.field(1).is_nullable());
-        assert!(!schema.field(2).is_nullable());
-        assert!(!schema.field(3).is_nullable());
+        assert!(schema.field(2).is_nullable());
+        assert!(schema.field(3).is_nullable());
 
         assert_eq!(&DataType::Int64, schema.field(0).data_type());
         assert_eq!(&DataType::Utf8, schema.field(1).data_type());
